<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\ObrigacaoLegal\Rais\GeraRais;
use HCM\FPA\ObrigacaoLegal\Rais\RetrocedeRais;
use Doctrine\ORM\EntityManager;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\Util\FPA;

class RAISController extends Crud {

    protected $RAISService;
    protected $dataSourceNameGet = 'getRAIS';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getRAIS (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                          
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['NOMEFINAL']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }   
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
    public function setRAIS(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_RAIS";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMRAIS', $row)) {
                    if ($row['NRFORMRAIS'] === '') {
                        $row['NRFORMRAIS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMRAIS', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMRAIS'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMRAIS', 1, 12, true); 
                }
            }
            if(!array_key_exists('DSINICIO', $row)){
                $row['DSINICIO'] = null;
            }

            if(!array_key_exists('DSFIM', $row)){
                $row['DSFIM'] = null;
            }
            if($row['DSORDEM'] === "LISTA_VINCULO"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
            }
            if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                if(is_array($row['DSINICIO'])){
                    $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                }
                if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                    $row['DSFIM'] = implode(',',$row['DSFIM']);
                }else{
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = null;
                    }
                }
            }
            
            $row['NRESTRUTRESPONS'] = isset($row['NRESTRUTRESPONS'])?$row['NRESTRUTRESPONS']:null;
            $row['NRPESSOARESPONS'] = isset($row['NRPESSOARESPONS'])?$row['NRPESSOARESPONS']:null;
            $row['NRESTRUTSIND'] = isset($row['NRESTRUTSIND'])?$row['NRESTRUTSIND']:null;
            $row['NOMEINICIAL'] = $row['DSINICIO'];
            $row['NOMEFINAL'] = isset($row['DSFIM']) ? $row['DSFIM'] : null;
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteRAIS(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_RAIS";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function geraRAIS(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            
            $idabort = false;
            $iderros = false;
            $idalert = false;
            
            $row = $request->getDataSet()->getRows()[0];
            
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrformrais = $row['NRFORMRAIS'];
            $grupoIncidencia = $row['NRGRPINCIDE'];
            $nrestrutsind = $row['NRESTRUTSIND'];
            
            $this->rais = new GeraRais($this->hcmParameters, $this->entityManager);
            $this->rais->setFormRais($nrformrais);
            $mensagens = $this->rais->gerar($grupoIncidencia, null, $nrestrutsind);
            $this->rais->preparaDados();
            $this->nmArquivo = $this->rais->preparaDownload();

            $return[] = array("NMARQUIVO" => $this->nmArquivo);
            foreach ($this->rais->getTipo00()->tipo01 as $tipo01) {
                if ($tipo01->idabort) {
                    $idabort = true;
                    $mensagens = $tipo01->mensagens;
                    break;
                }
                if ($tipo01->iderros) {
                    $iderros = true;
                }
                if ($tipo01->idalert) {
                    $idalert = true;
                }
            }
            if ($idabort) {
                $message = 'RAIS com erros. Não finalizado.<br>';
                foreach ($mensagens as $mensagem) {
                    $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - '.$mensagem['MENSAGEM'];
                }
                $response->setError(new DTO\Response\Error($message));
            } else {
                $row = array();
                foreach ($mensagens as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
                if ($iderros || $idalert) {
                    $nrcalculofolha = FPA::determinaNrCalculoFolha(FpatipomovimentEntity::TP_MOV_MENSAL);
                    if (!$iderros) {
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
                        $response->addNotification(new Notification("RAIS finalizado com advertências.<br>Verifique as mensagens.", Notification::TYPE_ALERT));
                    } else {
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
                        $response->addNotification(new Notification("RAIS finalizado com erros.<br>Verifique as mensagens.", Notification::TYPE_ALERT));
                    }
                } else {
                    $response->addNotification(new Notification("Arquivo gerado", Notification::TYPE_SUCCESS));
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
                }
            }
        } catch (\Exception $e) {
            throw $e/*->getMessage()*/;
        }   
    }
    public function downloadRAIS(DTO\Request\DataSet $request, DTO\Response $response){
    try{
            FPA::download($request);
        }
    catch (\Exception $e) {
                throw $e;
        } 
    }
    
    public function retrocedeRAIS(DTO\Request\DataSet $request, DTO\Response $response)
    {
    try{
        $row = $request->getDataSet()->getRows()[0];
        $this->retrocesso = new RetrocedeRais($this->hcmParameters, $this->entityManager);
        $this->retrocesso->retrocede($row['NRORG'],$this->hcmParameters->getDtCompetencia()); 
    }
    catch (\Exception $e) {
        throw $e;
    } 
    }
    
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}