<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\MovimentacaoService;
use HCM\Geral\Services\ESocialService;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Util;
use HCM\Util\Repositories;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use HCM\Util\DateUtil;

class MovimentacaoController extends Crud {

    protected $dataSourceName = 'gpe_movimentacao';
    protected $dataSourceNameGet = 'getMovimentacao';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, MovimentacaoService $movimentacaoService, EntityManager $entityManager, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->movimentacaoService = $movimentacaoService;
        $this->entityManager = $entityManager;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }

    public function getMovimentacao (DTO\Request\Filter $request, DTO\Response $response) {

        set_time_limit(0);
        ini_set('memory_limit', '2048M');
        
        set_time_limit(0);
        ini_set('memory_limit', '2048M');
        
        $filters = $request->getFilterCriteria()->getConditions();
        
        foreach ($filters as $key=>$filter) {
            if($filter['columnName'] === 'P_NRORG'){
                $nrorg = $filter['value'];
            }
            if($filter['columnName'] === 'P_NRORG_PADRAO'){
                $nrorg_padrao = $filter['value'];
            }
            if($filter['columnName'] === 'P_DTMESCOMPETENC'){
                $dtmescompetenc = $filter['value'];
            }
            if($filter['columnName'] === 'IDSOMENTECOMPET'){
                unset($filters[$key]);
            }
        }
        
        $orderBy = '';
        $orderByQuery = $request->getFilterCriteria()->getOrderBy(); 
        if($orderByQuery){
            foreach ($orderByQuery as $order => $value) {
                $orderBy .= 'ORDER BY '.$order.' '.$value;
            }
        }
        
        Util::getValueInFilterDataSourceByName($filters, array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC'), true);
        
        $conditions = Util::createQueryFiltered($filters);
        
        $dataSet = $this->movimentacaoService->getMovimentacaoCrud($nrorg, $nrorg_padrao, $dtmescompetenc, $conditions, $orderBy);
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getMovimentacao', $dataSet));
        
    }

    public function setMovimentacao (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (!isset($row['NRMOTIVOAFASTA'])) { $row['NRMOTIVOAFASTA'] = ''; }
                if (!isset($row['IDATUALIZAACERTO'])) { $row['IDATUALIZAACERTO'] = ''; }
                if (!isset($row['DSOBSERVACAO'])) { $row['DSOBSERVACAO'] = ''; }

                $nrorg = $row['NRORG'];
                $nrmovimentacao = isset($row['NRMOVIMENTACAO'])?$row['NRMOVIMENTACAO']:'';
                $nrvinculom = $row['NRVINCULOM'];
                $nrtipotransfer = $row['NRTIPOTRANSFER'];
                $nrtipoestrutura = $row['NRTIPOESTRUTURA'];
                $nrestruturam = $row['NRESTRUTURAM'];
                $dtinimoviment = $row['DTINIMOVIMENT'];
                $dtfimmoviment = isset($row['DTFIMMOVIMENT'])?$row['DTFIMMOVIMENT']:'';
                $iutilizamatric = array_key_exists('IDUTILIZAMATRIC', $row) ? $row['IDUTILIZAMATRIC'] : null;
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $isnew = $row['__is_new'];
                $row['NRMOTIVOTRANSF'] = array_key_exists('NRMOTIVOTRANSF', $row) ? $row['NRMOTIVOTRANSF'] : null;
                
                $this->movimentacaoService->verificaTipoTransferencia($nrtipotransfer, $nrtipoestrutura);
                $this->movimentacaoService->validaExisteDataMovimentacao($nrorg, $nrvinculom, $dtinimoviment, $dtfimmoviment, $nrtipoestrutura, $nrmovimentacao);
                $this->movimentacaoService->beforeSave($nrorg, $nrvinculom);

                if (array_key_exists('NRMOVIMENTACAO', $row)) {
                    if ($row['NRMOVIMENTACAO'] === '') {
                        $row['NRMOVIMENTACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true);
                    }
                } else {
                    $row['NRMOVIMENTACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true);
                }
            }
            
            parent::save($request, $response);
            
            $histVincEstrutAnte = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $this->hcmParameters->getDtCompAnterior());
            $matricula = $this->movimentacaoService->afterSave($nrorg, $nrvinculom, $nrestruturam, $nrtipoestrutura, $dtinimoviment, $dtfimmoviment, $iutilizamatric, $isnew);
            $matricula = $matricula ? $matricula : array('NOVO' => null);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('MatriculaESocial', $matricula));
           
            $retornoEsocial = [];
            $dataSetsResponse = [];
            
            if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                $estrutAnte = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($nrorg, $histVincEstrutAnte->getNrestrutlegal(), $dtmescompetenc);
             
               if(substr($row['CDCNPJESTRUT'], 1, 8) != substr($estrutAnte->getCdcnpjestrut(), 1, 8)){
                    $movSaida = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacaoTransfSaida($nrorg, $nrvinculom, $row['NRTIPOESTRUTURA'], $dtmescompetenc, $histVincEstrutAnte->getNrestrutlegal(), $this->hcmParameters->getDtCompAnterior(),  DateUtil::getUltimoDiaDoMes($dtmescompetenc))[0];
                    
                    // envio da movimentação de saída
                    $row['NMEVENTO'] .= '99';
                    $row['DTMESCOMPETENCIA'] = DateUtil::getPrimeiroDiaDoMes($movSaida->getDtfimmoviment())->format('d/m/Y'); 
                    $row['DTMESCOMPETENC'] = DateUtil::getPrimeiroDiaDoMes($movSaida->getDtfimmoviment())->format('d/m/Y'); 
                    $row['NRESTRUTURAM'] = $movSaida->getNrestruturam();
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                    
                    $dataSets = $response->getDataSets();
                    foreach($dataSets as $dataSet){
                        if($dataSet->getDataSourceName() == 'retornoEsocial'){
                            $retornoEsocial[$row['NMEVENTO']] = $dataSet->getRows()[0];
                        }else{
                            array_push($dataSetsResponse, $dataSet);        
                        }
                    }
                    $response->setDataSets($dataSetsResponse);
                    
                    // envio da movimentação de enntrada
                    $row['NMEVENTO'] = substr($row['NMEVENTO'], 0, 4).'00';
                    $row['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia(); 
                    $row['NRESTRUTURAM'] = $nrestruturam;
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                    
                    $dataSets = $response->getDataSets();
                    
                    
                    foreach($dataSets as $dataSet){
                        if($dataSet->getDataSourceName() == 'retornoEsocial'){
                            $retornoEsocial[$row['NMEVENTO']] = $dataSet->getRows()[0];
                        }else{
                            array_push($dataSetsResponse, $dataSet);        
                        }
                    }
                    
                    $response->setDataSets($dataSetsResponse);
                    $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retornoEsocial)));
                    
                }else if(substr($row['CDCNPJESTRUT'], 1, 8) == substr($estrutAnte->getCdcnpjestrut(), 1, 8)){
                    
                    $row['NMEVENTO'] .= '06'; 
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                    
                    $dataSets = $response->getDataSets();
                    
                    $dataSetsResponse = [];
                    foreach($dataSets as $dataSet){
                        if($dataSet->getDataSourceName() == 'retornoEsocial'){
                            $retornoEsocial[$row['NMEVENTO']] = $dataSet->getRows()[0];
                        }else{
                            array_push($dataSetsResponse, $dataSet);        
                        }
                    }
                    
                    $response->setDataSets($dataSetsResponse);
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retornoEsocial));
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteMovimentacao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrtipoestrutura = $row['NRTIPOESTRUTURA'];
                $nrestruturam = $row['NRESTRUTURAM'];
            }
            parent::delete($request, $response);
            $matricula = $this->movimentacaoService->atualizaEstruturaHistoricoVinculo($nrorg, $nrvinculom, $nrtipoestrutura, $nrestruturam, true);
            
            if($matricula){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('MatriculaESocial', $matricula));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function gerarMovimentacoes(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (!empty($row['NRTIPOTRANSFERORIG'])) {
                    $nrtipotransfer = $row['NRTIPOTRANSFERORIG'];    
                } else {
                    $nrtipotransfer = $row['NRTIPOTRANSFER'];
                }
                if (!empty($row['NRTIPOTRANSFERDEST'])) {
                    $nrtipotransferdestino = $row['NRTIPOTRANSFERDEST'];    
                }
                $nrtipoestruturaori = $row['NRTIPOESTRUTURAORIG'];
                $nrestruturamori = $row['NRESTRUTURAMORIG'];
                $vinculos = isset($row['NRVINCULOM'])?$row['NRVINCULOM']:array();
                $nrtpmovtransfm = $row['NRTPMOVTRANSFM'];
                $nrmotivotransf = isset($row['NRMOTIVOTRANSF'])?$row['NRMOTIVOTRANSF']:null;
                $dtinimoviment = $row['DTINIMOVIMENT'];
                $dtfimmoviment = isset($row['DTFIMMOVIMENT'])?$row['DTFIMMOVIMENT']:'';
                $idatualizaacerto = isset($row['IDATUALIZAACERTO'])?$row['IDATUALIZAACERTO']:'';
                $nrtipoestrutura = isset($row['NRTIPOESTRUTURA'])?$row['NRTIPOESTRUTURA']:'';
                $nrestruturam = $row['NRESTRUTURAM'];
                $dsobservacao = isset($row['DSOBSERVACAO'])?$row['DSOBSERVACAO']:'';

                $this->movimentacaoService->manutençãoMovimentaçãoVinculo($nrtipotransfer, $nrestruturamori, $nrtipoestruturaori, $vinculos, $nrtpmovtransfm,
                    $nrmotivotransf, $dtinimoviment, $dtfimmoviment, $idatualizaacerto, $nrtipoestrutura, $nrestruturam, $dsobservacao, $nrtipotransferdestino, $response);
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}