<?php

namespace HCM\Geral\Services;

use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Services\EstruturasService;
use HCM\Geral\Services\ParcnegocioService;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class EstruturacaoService {

    const ENDERECO_PRINCIPAL = 'PRINCIPAL';
    const FORMA_COMUNICACAO_TEL_PRINCIPAL = '01';
    const FORMA_COMUNICACAO_EMAIL = '05';

    private $nrorg;
    private $nrorgPadrao;
    private $mesCompetencia;
    private $ultimoDiaCompetencia;
    private $dtAux;
    private $estrutura;
    private $estruturaTomador;
    private $hierarquiaOrg;
    private $hierarquiaOrgTomador;
    private $tipoHierarquia;
    private $tipoHierarquiaTomador;
    private $movimentacao;
    private $movimentacaoVinculoResponsavel;
    private $movimentacoesVinculoTomador;

    private $areaRisco;
    private $vinculoAreaRisco;
    private $estruturaAreaRisco;

    private $enderecoParc;
    private $comunicaParc;
    private $municipio;
    private $bairro;
    private $listaDDD = array(
        "SP" => '011',
        "RJ" => '022',
        "ES" => '027',
        "MG" => '031',
        "PR" => '041',
        "SC" => '048',
        "RS" => '051',
        "DF" => '061',
        "GO" => '062',
        "TO" => '063',
        "MT" => '065',
        "MS" => '067',
        "AC" => '068',
        "RO" => '069',
        "BA" => '071',
        "SE" => '079',
        "PE" => '081',
        "AL" => '082',
        "PB" => '083',
        "RN" => '084',
        "CE" => '085',
        "PI" => '086',
        "PA" => '091',
        "AM" => '092',
        "RR" => '095',
        "AP" => '096',
        "MA" => '098',
    );

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, EstruturasService $estruturaService, ParcnegocioService $parcNegocioService){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters   = $hcmParameters;
        $this->estrutura = $estruturaService;
        $this->parcNegocioService = $parcNegocioService;
        $this->nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->mesCompetencia = Util::datadeString($this->hcmParameters->getDtCompetencia(), true);
        $this->ultimoDiaCompetencia = Util::datadeString($this->hcmParameters->getUltimoDiaMes(), true);
        $this->dtAux = Util::datadeString($this->hcmParameters->getDataAtual(), true);
        $this->hierarquiaOrg = new HierarquiaorgEntity($entityManager,$hcmParameters);
        $this->hierarquiaOrgTomador = new HierarquiaorgEntity($entityManager,$hcmParameters);
        $this->tipoHierarquia = new TipohierarqorgEntity($entityManager,$hcmParameters);
        $this->tipoHierarquiaTomador = new TipohierarqorgEntity($entityManager,$hcmParameters);
    }

    public function getAreaRisco(){
        return $this->areaRisco;
    }

    public function carregaHierarquias($nrTpHierarquiaOrg){
        $this->hierarquiaOrg->carregaHierarquiasPorTipo($this->nrorg, $nrTpHierarquiaOrg);
    }
    
    public function carregaHierarquiasTomador($nrTpHierarquiaOrgTomador){
        $this->hierarquiaOrgTomador->carregaHierarquiasPorTipo($this->nrorg, $nrTpHierarquiaOrgTomador);
    }

    public function carregaTipoHierarquia($nrTpHierarquiaOrg){
        $this->tipoHierarquia->carregaTipoHierarquia($this->nrorgPadrao, $this->nrorg, $nrTpHierarquiaOrg);
    }

    public function carregaTipoHierarquiaTomador($nrTpHierarquiaOrgTomador){
        $this->tipoHierarquiaTomador->carregaTipoHierarquia($this->nrorgPadrao, $this->nrorg, $nrTpHierarquiaOrgTomador);
    }

    public function getMovimentacaoVinculoResponsavel(){
        return $this->movimentacaoVinculoResponsavel;
    }
    
    public function getMovimentacaoVinculoTomador($posicao){
        return $this->movimentacoesVinculoTomador[$posicao];
    }

    public function retornaEstrutura($nrEstrutura) {
        return $this->estrutura->retornaEstrutura($nrEstrutura);
    }

    private function _verificaPeriodoValidoMes($data){
        $this->dtAux->set($this->mesCompetencia);
        $this->dtAux->addMonth(1)->subDay(1);
        $dtInicio = $this->mesCompetencia->toString(TkDateTime::FORMAT_DATE);
        $dtFim = $this->dtAux->toString(TkDateTime::FORMAT_DATE);

        if (empty($data) === false) {
            $existe = $this->verificaDataPeriodo($data, $dtInicio, $dtFim);
        } else {
            $existe = true;
        }

        return $existe;
    }

    private function _verificaDataPeriodo($data, $dtInicio, $dtFim){
        $this->dtAux->set($data);
        if ($this->dtAux->isLater($dtInicio) || $this->dtAux->equals($dtInicio)
           && $this->dtAux->isEarlier($dtFim) || $this->dtAux->equals($dtFim) ) {
            return true;
        } else {
            return false;
        }

    }

    public function incluiMovimentacaoVinculo(
        $nrTipoTransf, $nrVinculo, $nrEstrutura, $dtIniMoviment, $nrTipoEstrutura, $nrTpMovTransf, $dtRetroativa,
        $idAtualizacao, $dtFimMovimento
    ){
        $mov = $this->movimentacao->incluiMovimentacao(
            $this->nrorg, $nrTipoTransf, $nrVinculo, $nrEstrutura, $dtIniMoviment, $nrTipoEstrutura, $nrTpMovTransf, $dtRetroativa,
            $idAtualizacao, $dtFimMovimento
        );

        return $mov;
    }

    public function verificaExistenciaMovimentacao($nrTpMovTransferencia, $dtIniMovimento, $dtRetroativa){
        $verificaMovimentacao = $this->movimentacao->verificaMovPorTipoTransferencia(
            $nrTpMovTransferencia, $dtIniMovimento, $dtRetroativa
        );

        return $verificaMovimentacao;
    }

    public function getMovimentacoesVinculo($nrVinculo){
        $this->movimentacao->carregaMovimentacoes($this->nrorg, $nrVinculo, $this->ultimoDiaCompetencia);

        return $this->movimentacao->getItems();
    }

    public function getMovimentacoesVinculoPorTipo($nrVinculo, $tipo){
        $tipoEstrutura = \FPA::getTipoEstruturaLegalOrg();
        $this->movimentacao->carregaMovimentacoesPorTipo(
            $this->nrorg, $nrVinculo, $tipo, $this->ultimoDiaCompetencia, $tipoEstrutura
        );

        return $this->movimentacao->getItems();
    }

    public function getMovimentacaoVinculoPorTipo($nrVinculo, $tipo){
        $tipoEstrutura = \FPA::getTipoEstruturaLegalOrg();
        $this->movimentacao->carregaMovimentacoesPorTipo(
            $this->nrorg, $nrVinculo, $tipo, $this->ultimoDiaCompetencia, $tipoEstrutura
        );

        return current($this->movimentacao->getItems());
    }

    public function getEstruturas($nrVinculom, $nrTipoHierarquia){

        $paramsTipoHierarquia = array($this->nrorgPadrao, $this->nrorg, $nrTipoHierarquia);
        /** @var $tipoHierarquia TipohierarqorgEntity */
        $tipoHierarquia = TipohierarqorgEntity::getEntityByWhere(
            '(NRoRG = ? OR NRoRG = ?) AND NRTIPOHIERARQORG = ?', $paramsTipoHierarquia
        );

        $this->movimentacao->carregaMovimentacoesPorHierarquia($this->nrorg, $nrVinculom,
            $tipoHierarquia->getNrtipoestrutrel(), $this->mesCompetencia, $this->ultimoDiaCompetencia);

        foreach($this->movimentacao->getItems() as $item){
            $this->estrutura->carregaEstrutura($item->getNrestruturam());
            /** @var $estrutura EstruturamEntity */
            $estrutura = $this->estrutura->getEstrutura();
            if($estrutura){
                return $estrutura;
            }
        }
        return null;

    }

    public function getEstruturaSuperior($movimentacao){

        $hierarquias = $this->hierarquiaOrg->getItems();
        $tpHierarquia = $this->tipoHierarquia->getItem(0);

        if ((int)$movimentacao->getNrtipoestrutura() === (int)$tpHierarquia->getNrtipoestrutrel()) {
            $nrEstruturaSup = $this->determinaEstruturaSuperior(
                $hierarquias, $movimentacao->getNrestruturam(), $tpHierarquia->getIdautohierarq()
            );

            $this->estrutura->carregaEstrutura($nrEstruturaSup);
            $estrutura = $this->estrutura->getEstrutura();

            return $estrutura;
        }

    }

    public function getEstruturaSuperiorTomador($movimentacao){

        $hierarquiasTomador = $this->hierarquiaOrgTomador->getItems();
        $tpHierarquiaTomador = $this->tipoHierarquiaTomador->getItem(0);
        if ((int)$movimentacao->getNrtipoestrutura() === (int)$tpHierarquiaTomador->getNrtipoestrutrel()) {
            $nrEstruturaSup = $this->determinaEstruturaSuperior(
                $hierarquiasTomador, $movimentacao->getNrestruturam(), $tpHierarquiaTomador->getIdautohierarq()
            );

            $this->estruturaTomador->carregaEstrutura($nrEstruturaSup);
            return $this->estruturaTomador->getEstrutura();
        }

    }

    public function retornaEstruturaResponsavel($nrVinculo, $dtInicio, $dtFim){

        $hierarquias = $this->hierarquiaOrg->getItems();
        $tpHierarquia = $this->tipoHierarquia->getItem(0);

        //@todo Carregar todas as movimentacoes e testar dentro do foreach se a movimentacao esta na data corrente.
        $this->movimentacao->carregaMovimentacoesPeriodo($this->nrorg, $nrVinculo, $dtInicio, $dtFim);

        foreach($this->movimentacao->getItems() as $item){
            //@todo  - filtrar as movimentacoes por tipo de transferencia (legal / gerencial)
            if ((int)$item->getNrtipoestrutura() === (int)$tpHierarquia->getNrtipoestrutrel()) {
                 $nrEstruturaSup = $this->determinaEstruturaSuperior(
                    $hierarquias, $item->getNrestruturam(), $tpHierarquia->getIdautohierarq()
                );
                $this->estrutura->carregaEstrutura($nrEstruturaSup);
                $this->movimentacaoVinculoResponsavel = $item;
            }
        }

        return $this->estrutura->getItem(0);
    }

    public function retornaEstruturaTomador($nrVinculo, $dtInicio, $dtFim){
        $this->estruturaTomador->resetItems();
        $this->movimentacoesVinculoTomador = array();

        $hierarquiasTomador = $this->hierarquiaOrgTomador->getItems();
        $tpHierarquiaTomador = $this->tipoHierarquiaTomador->getItem(0);

        foreach($this->movimentacao->getItems() as $item){
            if ((int)$item->getNrtipoestrutura() === (int)$tpHierarquiaTomador->getNrtipoestrutrel()) {
                $nrEstruturaSup = $this->determinaEstruturaSuperior(
                    $hierarquiasTomador, $item->getNrestruturam(), $tpHierarquiaTomador->getIdautohierarq()
                );
                if ($nrEstruturaSup === null) {
                    MsgCalculo::getInstance()->adicionaMensagem('Verifique movimentação com estrutura nula para vinculo '.$nrVinculo);
                } else {
                    $this->estruturaTomador->carregaEstrutura($nrEstruturaSup);
                    $this->movimentacoesVinculoTomador[$nrEstruturaSup] = $item;
                }

            }

        }

        return $this->estruturaTomador->getItems();
    }

    private function _determinaEstruturaSuperior(array $hierarquias, $nrEstrutura, $autoHierarquia){
        // Metodo duplicado  - Contabilizacao. Ir para Hierarquia.
        if ($autoHierarquia === TipohierarqorgEntity::IDAUTOHIERARQ_SIM) {
            $nrEstruturaSup = $nrEstrutura;
        } else {
            $nrEstruturaSup = $this->hierarquiaOrg->determinaEstruturaSuperior($nrEstrutura, $hierarquias);
        }

        return $nrEstruturaSup;
    }

    public function determinaEnderecoEstrutura($estrutura){

        // Busca endereco da estrutura.
        $nrParceiro = $estrutura->historico->getNrparcnegocio();
        if ($nrParceiro) {

            $endereco = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findBy(array('nrorg' => $this->nrorg,
            'nrparcnegocio' => $nrParceiro,'cdtipoendereco'=> self::ENDERECO_PRINCIPAL))[0];

            if (!$endereco) {
                throw new \Exception('Conferir dados de endereço da estrutura '.$estrutura->historico->getNrestruturam().'.');
            }
            
            $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findBy(array('cdpais' => $endereco->getCdpais(),
            'sgestado' => $endereco->getSgestado(),'cdmunicipio'=> $endereco->getCdmunicipio()));

            if (is_object($municipio)) {
                $endereco->municipio = $municipio->getNmmunicipio();
            } else {
                $endereco->municipio = null;
            }

            // Bairro
            $endereco->bairro = $this->parcNegocioService->defineBairro($endereco);
            //$endereco->separaDsEndereco();

            // Complemento
            if ($endereco->getDscompleendereco() !== null){
                $endereco->complemento = $endereco->getDscompleendereco();
            } else if(method_exists($endereco, 'getComplemento')) {
                $endereco->complemento = $endereco->getComplemento();
            } else {
                $endereco->complemento = null;
            }

            $objComunicacoesParc = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(array('nrorg' => $this->nrorg,
            'nrparcnegocio' => $estrutura->historico->getNrparcnegocio()));
            
            $endereco->telefone = null;
            $endereco->email = null;
            $endereco->ddd = null;
            $endereco->ramal = null;

            foreach ($objComunicacoesParc as $formaComunicacao){

                /** @var $formaComunicacao ComunicaparcEntity **/

                // Telefone Principal.
                if ($formaComunicacao->getCdformacomu() ===  self::FORMA_COMUNICACAO_TEL_PRINCIPAL) {
                    $endereco->ddd = $formaComunicacao->getCdprefixcomuparc();
                    $endereco->telefone = $formaComunicacao->getCdcomunicaparc();
                    $endereco->ramal = '99999999';
                }

                // E-mail.
                if ($formaComunicacao->getCdformacomu() ===  self::FORMA_COMUNICACAO_EMAIL) {
                    $endereco->email = $formaComunicacao->getCdcomunicaparc();
                }
            }

            return $endereco;

        } else {
            return null;
        }
    }


    public function determinaAreaRisco($vinculo, $estrutura){
        // Verifica se o vinculo esta associado a alguma area de risco (AR).
        $resultados = $this->vinculoAreaRisco->retornaVinculoAreaRisco($this->nrorg, $vinculo->getNrvinculom());
        foreach ($resultados as $item) {
            $periodoValido = $this->verificaPeriodoValidoMes($item->getDtfimvincarearisco());
            if ($periodoValido === true) {
                // Descobre a AR associada ao vinculo
                $areaRisco = $this->areaRisco->retornaAreaRisco(
                    $this->nrorg, $this->mesCompetencia, $item->getNrareariscom()
                );
            }
        }

        // Se vinculo nao estiver associado a AR verifica a associacao da estrutura com a AR.
        if (is_object($areaRisco) === false) {
            //
            $resultados = $this->estruturaAreaRisco->retornaEstruturaAreaRisco(
                $this->nrorg, $estrutura->getNrestruturam()
            );
            foreach ($resultados as $item) {
                $periodoValido = $this->verificaPeriodoValidoMes($item->getDtfimestrutarearisco());
                if ($periodoValido === true) {
                    // Descobre a AR associada a estrutura
                    $areaRisco = $this->areaRisco->retornaAreaRisco(
                        $this->nrorg, $this->mesCompetencia, $item->getNrareariscom()
                    );
                }
            }
        }

        $this->areaRisco = $areaRisco;

    }

    public function getDDD($sgestado)
    {
        return $this->listaDDD[$sgestado];
    }

}