<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\AcaoService;
use HCM\Util\Util;

class AcaoController extends Crud {
    
    protected $dataSourceName;
    protected $dataSourceNameAcao = 'sst_acao';
    protected $dataSourceNameTramitacao = 'sst_tramitacao';
    protected $dataSourceNameLogprevacao = 'sst_logprevacao';
    protected $dataSourceNameEtapaAcao = 'sst_etapaacao';
    protected $dataSourceNameObsAcao = 'sst_obsacao';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AcaoService $acaoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->AcaoService = $acaoService;
        parent::__construct($dataSourceManager);
    }

    public function getAcao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("sst_acao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setAcao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameAcao;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRACAO', $row)) {
                    if ($row['NRACAO'] === '') {
                        $row['NRACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ACAO', 1, 12, true);
                    }
                } else {
                    $row['NRACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ACAO', 1, 12, true);
                }
            
             $operador = $this->hcmParameters->getCdoperador();
             $acao = $row['NRACAO'];
             $row["CDFILIAL"] = isset($row["CDFILIAL"])?$row["CDFILIAL"]:'0001';
             $row["CDCLIENTE"] = isset($row["CDCLIENTE"])?$row["CDCLIENTE"]:null;
             $row["CDFORNECED"] = isset($row["CDFORNECED"])?$row["CDFORNECED"]:null;
             $row["CDVENDEDOR"] = isset($row["CDVENDEDOR"])?$row["CDVENDEDOR"]:null;
             $row["CDPROSPECT"] = isset($row["CDPROSPECT"])?$row["CDPROSPECT"]:null;
             $row["CDCLAUSULA"] = isset($row["CDCLAUSULA"])?$row["CDCLAUSULA"]:null;
             $row["CDGRAUACAO"] = isset($row["CDGRAUACAO"])?$row["CDGRAUACAO"]:null;
             $row["CDOPERABERAC"] = isset($row["CDOPERRESPAC"])?$row["CDOPERRESPAC"]:null;
             $row["DTPREVINICAC"] = isset($row["DTPREVINICAC"])?$row["DTPREVINICAC"]:null;
             $row["DTPREVENCEAC"] = isset($row["DTPREVENCEAC"])?$row["DTPREVENCEAC"]:null;
             $dtprevenceac = $row["DTPREVENCEAC"];
             $row["DTINICACAO"] = isset($row["DTINICACAO"])?$row["DTINICACAO"]:null;
             $row["DTENCEACAO"] = isset($row["DTENCEACAO"])?$row["DTENCEACAO"]:null;
             $row["IDACAOENCE"] = isset($row["IDACAOENCE"])?$row["IDACAOENCE"]:'N';
             $row["IDTPMOTIVADO"] = isset($row["IDTPMOTIVADO"])?$row["IDTPMOTIVADO"]:null;
             $row["CDOPERAVAL"] = isset($row["CDOPERAVAL"])?$row["CDOPERAVAL"]:null;
             $row["DTPRAVALACAO"] = isset($row["DTPRAVALACAO"])?$row["DTPRAVALACAO"]:null;
             $row["IDACAOLIDA"] = isset($row["IDACAOLIDA"])?$row["IDACAOLIDA"]:null;
             $row["CDSETORESPAC"] = isset($row["CDSETORESPAC"])?$row["CDSETORESPAC"]:null;
             $row["DTULTATUACAO"] = isset($row["DTULTATUACAO"])?$row["DTULTATUACAO"]:null;
             $row["CDSETOABERAC"] = isset($row["CDSETOABERAC"])?$row["CDSETOABERAC"]:null;
             $row["NRTAREFA"] = isset($row["NRTAREFA"])?$row["NRTAREFA"]:null;
             $row["NRAUDITPROC"] = isset($row["NRAUDITPROC"])?$row["NRAUDITPROC"]:null;
             $row["IDATIVO"] = isset($row["IDATIVO"])?$row["IDATIVO"]:"S";
             $row['DTULTATU'] = isset($row['DTULTATU'])?$row['DTULTATU']:$row['DTINCLUSAO'];
             $salvaH = $row['salvaH'];
             $nrorg = $row['NRORG'];
             $dtultatu = $row['DTULTATU'];
            }
            parent::save($request, $response);
            if($salvaH){
                $this->AcaoService->salvaHistorico($nrorg,'0001',$acao,$dtultatu,$operador,$dtprevenceac);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteAcao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameAcao;
                foreach ($request->getDataSet()->getRows() as &$row) {
                $res = $this->AcaoService->validaDelete($row['NRACAO'],$row['NRORG']);
            }
            if($res)
                parent::delete($request, $response);
            else {
                throw new \Exception("Não foi possivel excluir, existem itens relacionados a este registro", 10);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getTramitacao(DTO\Request\Filter $request, DTO\Response $response) {

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("sst_tramitacao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getLogprevacao(DTO\Request\Filter $request, DTO\Response $response) {

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("sst_logprevacao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setLogprevacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameLogprevacao;
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $row["CDFILIAL"] = isset($row["CDFILIAL"])?$row["CDFILIAL"]:'0001';
                $row["IDATIVO"] = isset($row["IDATIVO"])?$row["IDATIVO"]:"S";
                $row["DSALPREVACAO"] = isset($row["DSALPREVACAO"])?$row["DSALPREVACAO"]:null;
                
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteLogprevacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameLogprevacao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function setTramitacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameTramitacao;
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $row["CDFILIAL"] = isset($row["CDFILIAL"])?$row["CDFILIAL"]:'0001';
                $row["DTALTRAMACAO"] = $row["DTINCLUSAO"];
                $row["DSALTRAMACAO"] = isset($row["DSALTRAMACAO"])?$row["DSALTRAMACAO"]:null;
                $row["CDSETRECEBAC"] = isset($row["CDSETRECEBAC"])?$row["CDSETRECEBAC"]:null;
                $row["IDATIVO"] = isset($row["IDATIVO"])?$row["IDATIVO"]:"S";
                
                // $this->AcaoService->setFaseTramitaAcao($row['NRORG'], $row['NRACAO'], $row['CDFATRAMACAO']);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteTramitacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameTramitacao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getOperadorResponsavel(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $position => $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                    }
                }
                if($filter['columnName'] == "P_PARECERFINAL"){
                    unset($filters[$position]);
                }
            }
            Util::getValueInFilterDataSourceByName($filters, array('P_NRORG'), true);
            $conditions = Util::createQueryFiltered($filters);
            $result = $this->AcaoService->getOperadorResponsavel($parameters, $conditions);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getOperadorResponsavel', $result));
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getEtapaAcao(DTO\Request\Filter $request, DTO\Response $response) {

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("sst_etapaacao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setEtapaAcao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameEtapaAcao;
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                if (array_key_exists('NRETAPACAO', $row)) {
                    if ($row['NRETAPACAO'] === '') {
                        $row['NRETAPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ACAO', 1, 12, true);
                    }
                } else {
                    $row['NRETAPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'ACAO', 1, 12, true);
                }
                $row["IDATIVO"] = isset($row["IDATIVO"])?$row["IDATIVO"]:"A";
                $row["CDFILIAL"] = isset($row["CDFILIAL"])?$row["CDFILIAL"]:'0001';
                $row["IDSITUETAPACAO"] = isset($row["IDSITUETAPACAO"])?$row["IDSITUETAPACAO"]:"P";
                $row["DSETAPACAO"] = isset($row["DSETAPACAO"])?$row["DSETAPACAO"]:null;
                
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEtapaAcao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameEtapaAcao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getObsAcao(DTO\Request\Filter $request, DTO\Response $response) {

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("sst_obsacao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setObsAcao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameObsAcao;

            foreach ($request->getDataSet()->getRows() as &$row) {
                $row["IDATIVO"] = isset($row["IDATIVO"])?$row["IDATIVO"]:"A";
                $row["CDFILIAL"] = isset($row["CDFILIAL"])?$row["CDFILIAL"]:'0001';
                $row['DTHROBSACAO'] = (isset($row['DTHROBSACAO']) && $row['DTHROBSACAO'] != "")?$row['DTHROBSACAO']:$row['DTINCLUSAO'];
                $row["CDOPEVISTOAC"] = isset($row["CDOPEVISTOAC"])?$row["CDOPEVISTOAC"]:null;
                $row['DSOBSERVAACAO'] = $row['DSOBSACAO']; 
                $row['DSOBSERVAACAO'] = base_convert($row['DSOBSERVAACAO'], 16, 2); // converte para binário
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteObsAcao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceNameObsAcao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 